<?php

namespace Chekonline\CashboxModule\Controller\Adminhtml\System;


use Magento\Backend\App\Action;
use Magento\Backend\App\Action\Context;
use Magento\Framework\Controller\Result\JsonFactory;
use Chekonline\Cashbox\Exceptions\ChekonlineCommandException;
use Chekonline\Cashbox\Exceptions\ChekonlineLineException;
use Chekonline\Cashbox\Exceptions\ChekonlineSDKException;
use Chekonline\CashboxModule\Helper\Data;
use Chekonline\Cashbox\Api;
use Chekonline\Cashbox\Commands\BatchCommand;
use Chekonline\Cashbox\Commands\NoOperationCommand;
use Magento\Framework\AppInterface;
use Chekonline\CashboxModule\Helper\ChekonlineCashboxHelper;

class Index extends Action
{

    /**
     * @var array
     */
    protected $_publicActions = ['index'];

    /**
     * @var JsonFactory
     */
    protected $resultJsonFactory;

    /**
     * @var Api
     */
    protected $chekonlineApi;

    /**
     * @var Data
     */
    protected $helper;

    /**
     * @param Context $context
     * @param JsonFactory $resultJsonFactory
     * @param Data $helper
     */
    public function __construct(
        Context $context,
        JsonFactory $resultJsonFactory,
        Data $helper
    )
    {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->helper = $helper;


        $this->chekonlineApi = new Api($this->helper->getGeneralConfig('endpoint'));
        $this->chekonlineApi->setClientName('Magento');
        $this->chekonlineApi->setClientVer($this->helper->getMagentoVersion());
        if ($this->helper->getGeneralConfig('cert'))
        {
            $this->chekonlineApi->setCertificate($this->helper->getGeneralConfig('cert'),
                $this->helper->getGeneralConfig('key'),
                $this->helper->getGeneralConfig('cert_password')
            );
        }

        parent::__construct($context);
    }

    /**
     * @return \Magento\Framework\Controller\Result\Json
     */
    public function execute()
    {

        $result = $this->resultJsonFactory->create();
        try {
            $requestId = strval(time());
            $batchCommand = new BatchCommand($requestId);
            $batchCommand->setClientId($this->helper->getGeneralConfig('client_id'));

            $batchCommand->setQueueLen(100)->setDevice('auto');


            $noOperationCommand = new NoOperationCommand($requestId);
            $noOperationCommand->setClientId($this->helper->getGeneralConfig('client_id'));

            $noOperationCommand->setPassword(intval($this->helper->getGeneralConfig('cashier_password')));

            $batchCommand->addCommandForExecute($noOperationCommand);


            $noOperationCommand->validate();
            $batchCommand->validate();
            $this->chekonlineApi->validate();

            $apiResponse =  $this->chekonlineApi->executeCommand($batchCommand);



            if ($this->isReceiptSuccess($apiResponse) == true) {
                $result = $result->setData(['Соединение установлено']);
            } else {
                $result = $result->setData(["Код ответа: {$apiResponse->getApiErrorCode()}\nОтвет: {$apiResponse->getApiErrorMessage()}"]);
            }
//            $result = $result->setData([$apiResponse->getRequest()]);
//            return $result;

            if ($this->helper->getGeneralConfig('log')) {

                ChekonlineCashboxHelper::writeLog('request', $apiResponse->getRequest());
                ChekonlineCashboxHelper::writeLog('response', $apiResponse);
            }

            if ($this->isReceiptSuccess($apiResponse) == false) {
                ChekonlineCashboxHelper::writeLog('error', $apiResponse);
            }

        } catch (ChekonlineLineException $exception) {
            $logMessage = "Chekonline line receipt exception:\n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "\nLine:\n".var_export($exception->getReceiptLine()->getParam(),true);
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            ChekonlineCashboxHelper::writeLog('fail', $logMessage);
            $result->setData([$exception->getMessage()]);
        } catch (ChekonlineCommandException $exception) {
            $logMessage = "Chekonline command exception: \n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "\nCommand:".var_export($exception->getCommand()->getParams(),true);
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            ChekonlineCashboxHelper::writeLog('fail', $logMessage);
            $result->setData([$exception->getMessage()]);
        } catch (ChekonlineSDKException $exception) {
            $logMessage = "Chekonline exception: \n\tMessage: {$exception->getMessage()}";
            $logMessage .= "\n\tAt: {$exception->getFile()}\n";
            $logMessage .= "Trace:\n{$exception->getTraceAsString()}\n";
            ChekonlineCashboxHelper::writeLog('fail', $logMessage);
            $result->setData([$exception->getMessage()]);
        }
        /** @var \Magento\Framework\Controller\Result\Json $result */
        return $result;
    }

    /**
     * @param \Chekonline\Cashbox\ChekonlineResponse $apiResponse
     * @return bool
     */
    private function isReceiptSuccess($apiResponse)
    {
        if ($apiResponse->getHttpStatusCode() == 200 &&
            $apiResponse->getApiErrorCode() == 0 &&
            $apiResponse->getHttpError() == '') {
            return true;
        } else {
            return false;
        }
    }
}